@@ -1,23 +1,9 @@
 /*
+ * oFono - Open Source Telephony
+ * Copyright (C) 2011-2012  Intel Corporation
+ * Copyright (C) 2017  sysmocom s.f.m.c. GmbH <info@sysmocom.de>
  *
- *  oFono - Open Source Telephony
- *
- *  Copyright (C) 2011-2012  Intel Corporation. All rights reserved.
- *  Copyright (C) 2017 by sysmocom s.f.m.c. GmbH <info@sysmocom.de>
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
+ * SPDX-License-Identifier: GPL-2.0-only
  */
 
 #ifdef HAVE_CONFIG_H
@@ -38,8 +24,6 @@
 
 struct ussd_data {
 	struct qmi_service *voice;
-	uint16_t major;
-	uint16_t minor;
 };
 
 static int validate_ussd_data(const struct qmi_ussd_data *data, uint16_t size)
@@ -159,51 +143,25 @@ error:
 	ofono_ussd_notify(ussd, OFONO_USSD_STATUS_TERMINATED, 0, NULL, 0);
 }
 
-static void create_voice_cb(struct qmi_service *service, void *user_data)
-{
-	struct ofono_ussd *ussd = user_data;
-	struct ussd_data *data = ofono_ussd_get_data(ussd);
-
-	DBG("");
-
-	if (service == NULL) {
-		ofono_error("Failed to request Voice service");
-		ofono_ussd_remove(ussd);
-		return;
-	}
-
-	if (!qmi_service_get_version(service, &data->major, &data->minor)) {
-		ofono_error("Failed to get Voice service version");
-		ofono_ussd_remove(ussd);
-		return;
-	}
-
-	data->voice = service;
-
-	qmi_service_register(data->voice, QMI_VOICE_USSD_IND,
-					async_ind, ussd, NULL);
-
-	qmi_service_register(data->voice, QMI_VOICE_ASYNC_ORIG_USSD,
-					async_orig_ind, ussd, NULL);
-
-	ofono_ussd_register(ussd);
-}
-
 static int qmi_ussd_probe(struct ofono_ussd *ussd,
 				unsigned int vendor, void *user_data)
 {
-	struct qmi_device *device = user_data;
+	struct qmi_service *voice = user_data;
 	struct ussd_data *data;
 
 	DBG("");
 
 	data = l_new(struct ussd_data, 1);
+	data->voice = voice;
+
+	qmi_service_register(data->voice, QMI_VOICE_USSD_IND,
+					async_ind, ussd, NULL);
+	qmi_service_register(data->voice, QMI_VOICE_ASYNC_ORIG_USSD,
+					async_orig_ind, ussd, NULL);
+
 
 	ofono_ussd_set_data(ussd, data);
 
-	qmi_service_create_shared(device, QMI_SERVICE_VOICE,
-						create_voice_cb, ussd, NULL);
-
 	return 0;
 }
 
@@ -216,7 +174,6 @@ static void qmi_ussd_remove(struct ofono_ussd *ussd)
 	ofono_ussd_set_data(ussd, NULL);
 
 	qmi_service_free(data->voice);
-
 	l_free(data);
 }
 
@@ -312,6 +269,7 @@ error:
 }
 
 static const struct ofono_ussd_driver driver = {
+	.flags		= OFONO_ATOM_DRIVER_FLAG_REGISTER_ON_PROBE,
 	.probe		= qmi_ussd_probe,
 	.remove		= qmi_ussd_remove,
 	.request	= qmi_ussd_request,
