@@ -1,22 +1,8 @@
 /*
+ * oFono - Open Source Telephony
+ * Copyright (C) 2008-2013  Intel Corporation
  *
- *  oFono - Open Source Telephony
- *
- *  Copyright (C) 2008-2013  Intel Corporation. All rights reserved.
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
+ * SPDX-License-Identifier: GPL-2.0-only
  */
 
 #ifndef __OFONO_SIRI_H
@@ -26,6 +12,8 @@
 extern "C" {
 #endif
 
+#include <stdarg.h>
+
 #include <ofono/types.h>
 
 struct ofono_siri;
@@ -34,8 +22,10 @@ typedef void (*ofono_siri_cb_t)(const struct ofono_error *error,
 					struct ofono_siri *siri);
 
 struct ofono_siri_driver {
-	const char *name;
+	unsigned int flags;
 	int (*probe)(struct ofono_siri *siri, unsigned int vendor, void *data);
+	int (*probev)(struct ofono_siri *siri, unsigned int vendor,
+			va_list args);
 	void (*remove)(struct ofono_siri *siri);
 	void (*set_eyes_free_mode) (struct ofono_siri *siri, ofono_siri_cb_t cb,
 					unsigned int val);
@@ -43,12 +33,9 @@ struct ofono_siri_driver {
 
 void ofono_siri_set_status(struct ofono_siri *siri, int value);
 
-int ofono_siri_driver_register(const struct ofono_siri_driver *driver);
-void ofono_siri_driver_unregister(const struct ofono_siri_driver *driver);
-
 struct ofono_siri *ofono_siri_create(struct ofono_modem *modem,
 					unsigned int vendor,
-					const char *driver, void *data);
+					const char *driver, ...);
 
 void ofono_siri_register(struct ofono_siri *siri);
 void ofono_siri_remove(struct ofono_siri *siri);
